"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const commons_1 = require("@git-temporal/commons");
const logger_1 = require("@git-temporal/logger");
const { debug, error } = logger_1.createProxies('git-diff-scraper');
function getDiff(requestPath = '.', leftCommit, rightCommit) {
    const _leftCommit = leftCommit || 'HEAD';
    const _rightCommit = rightCommit || 'local';
    const gitRoot = commons_1.findGitRoot(requestPath);
    const normalizedRequestPath = normalizeRequestPath(gitRoot, requestPath);
    debug('getDiff', requestPath, _leftCommit, _rightCommit);
    const { contents: leftFileContents, isDirectory } = fetchContents(_leftCommit, requestPath, gitRoot);
    const { contents: rightFileContents } = fetchContents(_rightCommit, requestPath, gitRoot);
    const modifiedFiles = isDirectory &&
        fetchDirectoryDiff(leftCommit, rightCommit, normalizedRequestPath, gitRoot);
    return {
        isDirectory,
        leftFileContents,
        rightFileContents,
        modifiedFiles,
        rightCommit: _rightCommit,
        leftCommit: _leftCommit,
        path: requestPath,
    };
}
exports.getDiff = getDiff;
function fetchContents(commitId, requestPath, gitRoot) {
    debug('fetchContents', { commitId, requestPath, gitRoot });
    return commitId === 'local'
        ? fetchFromLocal(requestPath)
        : fetchFromGit(commitId, requestPath === '.' ? './' : requestPath, gitRoot);
}
function fetchFromGit(commitId, requestPath, gitRoot) {
    const directory = path_1.default.dirname(requestPath);
    const baseFileName = path_1.default.basename(requestPath);
    const pathPrefix = directory === gitRoot ? '' : `./`;
    const cmd = `show ${commitId}:${pathPrefix}${commons_1.escapeForCli(baseFileName)}`;
    const rawContents = execGit(directory, cmd);
    const isDirectory = rawContents.match(/^tree /) !== null;
    return {
        isDirectory,
        contents: isDirectory ? null : Buffer.from(rawContents).toString('base64'),
    };
}
function fetchFromLocal(requestPath) {
    debug('fetchFromLocal', { requestPath });
    if (!fs_1.default.existsSync(requestPath)) {
        return { contents: null, isDirectory: false };
    }
    const isDirectory = fs_1.default.statSync(requestPath).isDirectory();
    const returnValue = {
        isDirectory,
        contents: null,
    };
    if (!isDirectory) {
        returnValue.contents = fs_1.default.readFileSync(requestPath).toString('base64');
    }
    return returnValue;
}
function fetchDirectoryDiff(_leftCommit, _rightCommit, requestPath, gitRoot) {
    const path = requestPath === '.' ? './' : requestPath;
    const leftCommit = _leftCommit || 'HEAD';
    const rightCommit = _rightCommit || 'local';
    const isDiffOnLocal = rightCommit === 'local';
    const leftPath = isDiffOnLocal ? leftCommit : `${leftCommit}:${path}`;
    const rightPath = isDiffOnLocal ? path : `${rightCommit}:${path}`;
    const extraOpts = '--stat=300 --compact-summary';
    let outputLines = [];
    try {
        const outputBuffer = execGit(gitRoot, `diff ${extraOpts} ${leftPath} ${rightPath}`);
        outputLines = outputBuffer.toString().split('\n');
    }
    catch (e) {
        error('Error retrieving git diff', (e.stderr && e.stderr.toString()) || e.toString());
    }
    return parseDirectoryDiff(outputLines);
}
function parseDirectoryDiff(outputLines) {
    const modifiedFiles = [];
    for (const line of outputLines) {
        let matches = line.match(/(.*)\((gone|new)\).*\|\s*(\d*)/);
        debug('parsing dir diff line', { line, matches });
        if (matches) {
            const [fileName, newOrGone, delta] = matches.slice(1);
            const status = newOrGone === 'new' ? 'added' : 'deleted';
            modifiedFiles.push(makeFile(fileName, delta, status));
        }
        else {
            matches = line.match(/^([^|]*)\|\s*(\d*)/);
            if (matches) {
                const [fileName, delta] = matches.slice(1);
                modifiedFiles.push(makeFile(fileName, delta));
            }
        }
    }
    return modifiedFiles.sort(fileNameComparator);
}
function makeFile(fileName, delta, status = 'modified') {
    return {
        status,
        path: fileName.trim(),
        delta: parseInt(delta, 10),
    };
}
function fileNameComparator(a, b) {
    return a.path.localeCompare(b.path);
}
function normalizeRequestPath(gitRoot, requestPath) {
    if (!gitRoot) {
        return requestPath;
    }
    if (gitRoot === requestPath) {
        return './';
    }
    const parsedRequestPath = path_1.default.parse(requestPath);
    if (parsedRequestPath.root === '') {
        return requestPath;
    }
    const relativeDir = parsedRequestPath.dir.slice(gitRoot.length + 1);
    debug(`normalizeRequestPath ${gitRoot},
      ${relativeDir},
      ${parsedRequestPath.dir},
      ${parsedRequestPath.base}`);
    return path_1.default.join(relativeDir, parsedRequestPath.base);
}
function execGit(gitRoot, gitCmd) {
    return commons_1.execSync(`git ${gitCmd}`, { cwd: gitRoot, logFn: debug });
}
